package rocketchat

import (
	"context"
	"crypto/sha256"
	"encoding/hex"
	"fmt"
	"log"
	"time"

	"go.mongodb.org/mongo-driver/bson"
	"go.mongodb.org/mongo-driver/bson/primitive"
	"go.mongodb.org/mongo-driver/mongo"
	"golang.org/x/crypto/bcrypt"
)

func (rci *RocketChatInstance) RocketChatRestAdminPass(client *mongo.Client, user, pass string) error {
	collection := client.Database("rocketchat").Collection("users")

	sha := sha256.Sum256([]byte(pass))
	hexSha := hex.EncodeToString(sha[:])
	hashedPass, err := bcrypt.GenerateFromPassword([]byte(hexSha), bcrypt.DefaultCost)
	if err != nil {
		return err
	}

	filter := bson.D{{"username", user}}
	update := bson.D{
		{"$set", bson.D{
			{"services", bson.D{{"password", bson.D{{"bcrypt", string(hashedPass)}}}}},
		}},
	}

	_, e := collection.UpdateOne(context.TODO(), filter, update)
	if e != nil {
		return e
	}

	log.Printf("%s's password changed", user)
	return nil
}

func (rci *RocketChatInstance) RocketChatUserCount(client *mongo.Client, active bool, lastLogin int) error {
	collection := client.Database("rocketchat").Collection("users")
	filter := bson.D{}
	if active {
		filter = append(filter, primitive.E{"active", true})
	}
	if lastLogin > 0 {
		filter = append(filter, primitive.E{"lastLogin", primitive.D{
			{"$lte", time.Now()},
			{"$gte", time.Now().AddDate(0, 0, -lastLogin)},
		}})
	}

	count, err := collection.CountDocuments(context.TODO(), filter)
	if err != nil {
		return err
	}

	fmt.Printf("%d users", count)
	return nil
}
